@@ -115,6 +115,11 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 
 	errno = 0;
 	if ( ! XauWriteAuth (af, xa)) {
+		free (xa->data);
+		free (xa->number);
+		free (xa->name);
+		free (xa->address);
+		free (xa);
 		display_add_error (d);
 		return FALSE;
 	}
@@ -122,6 +127,11 @@ add_auth_entry (GdmDisplay *d, FILE *af, FILE *af2,
 	if (af2 != NULL) {
 		errno = 0;
 		if ( ! XauWriteAuth (af2, xa)) {
+			free (xa->data);
+			free (xa->number);
+			free (xa->name);
+			free (xa->address);
+			free (xa);
 			display_add_error (d);
 			return FALSE;
 		}
@@ -145,8 +155,7 @@ gboolean
 gdm_auth_secure_display (GdmDisplay *d)
 {
     FILE *af, *af_gdm;
-    struct hostent *hentry = NULL;
-    struct in_addr *ia = NULL;
+    gboolean is_local = FALSE;
     const char lo[] = {127,0,0,1};
     guint i;
     const GList *local_addys = NULL;
@@ -197,11 +206,13 @@ gdm_auth_secure_display (GdmDisplay *d)
 	    /* Make another authfile since the greeter can't read the server/user
 	     * readable file */
 	    d->authfile_gdm = g_strconcat (GdmServAuthDir, "/", d->name, ".Xauth", NULL);
-	    unlink (d->authfile_gdm);
-
-	    af_gdm = fopen (d->authfile_gdm, "w");
+	    af_gdm = gdm_safe_fopen_w (d->authfile_gdm);
 
 	    if (af_gdm == NULL) {
+		    gdm_error (_("%s: Cannot safely open %s"),
+			       "gdm_auth_user_remove",
+			       d->authfile_gdm);
+
 		    g_free (d->authfile_gdm);
 		    d->authfile_gdm = NULL;
 		    g_free (d->authfile);
@@ -212,11 +223,13 @@ gdm_auth_secure_display (GdmDisplay *d)
     } else {
 	    /* gdm and xserver authfile can be the same, server will run as root */
 	    d->authfile = g_strconcat (GdmServAuthDir, "/", d->name, ".Xauth", NULL);
-	    unlink (d->authfile);
-
-	    af = fopen (d->authfile, "w");
+	    af = gdm_safe_fopen_w (d->authfile);
 
 	    if (af == NULL) {
+		    gdm_error (_("%s: Cannot safely open %s"),
+			       "gdm_auth_user_remove",
+			       d->authfile);
+
 		    g_free (d->authfile);
 		    d->authfile = NULL;
 		    return FALSE;
@@ -258,23 +271,22 @@ gdm_auth_secure_display (GdmDisplay *d)
 		    d->hostname = g_strdup (hostname);
 	    }
 	    local_addys = gdm_peek_local_address_list ();
-    } else  {
-	    /* Find FQDN or IP of display host */
-	    hentry = gethostbyname (d->hostname);
 
-	    if (hentry == NULL) {
-		    gdm_error ("gdm_auth_secure_display: Error getting hentry for %s", d->hostname);
-		    /* eeek, there will be nothing to add */
-		    return FALSE;
-	    } else {
-		    /* first addy, would be loopback in case of local */
-		    ia = (struct in_addr *) hentry->h_addr_list[0];
+	    is_local = TRUE;
+    } else  {
+	    is_local = FALSE;
+	    if (gdm_is_local_addr (&(d->addr)))
+		    is_local = TRUE;
+	    for (i = 0; ! is_local && i < d->addr_count; i++) {
+		    if (gdm_is_local_addr (&(d->addrs[i]))) {
+			    is_local = TRUE;
+			    break;
+		    }
 	    }
     }
 
     /* Local access also in case the host is very local */
-    if (SERVER_IS_LOCAL (d) ||
-	(ia != NULL && gdm_is_local_addr (ia))) {
+    if (is_local) {
 	    gdm_debug ("gdm_auth_secure_display: Setting up socket access");
 
 	    if ( ! add_auth_entry (d, af, af_gdm, FamilyLocal,
@@ -310,14 +322,22 @@ gdm_auth_secure_display (GdmDisplay *d)
     }
 
     gdm_debug ("gdm_auth_secure_display: Setting up network access");
+
+    if ( ! SERVER_IS_LOCAL (d)) {
+	    /* we should write out an entry for d->addr since
+	       possibly it is not in d->addrs */
+	    if ( ! add_auth_entry (d, af, af_gdm, FamilyInternet,
+				   (char *)&(d->addr), 4))
+		    return FALSE;
+    }
     
     /* Network access: Write out an authentication entry for each of
      * this host's official addresses */
-    for (i = 0 ; hentry != NULL && i < hentry->h_length ; i++) {
-	    ia = (struct in_addr *) hentry->h_addr_list[i];
+    for (i = 0; i < d->addr_count; i++) {
+	    struct in_addr *ia = &(d->addrs[i]);
 
-	    if (ia == NULL)
-		    break;
+	    if (memcmp (ia, &(d->addr), sizeof (struct in_addr)) == 0)
+		    continue;
 
 	    if ( ! add_auth_entry (d, af, af_gdm, FamilyInternet,
 				   (char *)&ia->s_addr, 4))
@@ -327,7 +347,7 @@ gdm_auth_secure_display (GdmDisplay *d)
     /* Network access: Write out an authentication entry for each of
      * this host's local addresses if any */
     for (; local_addys != NULL; local_addys = local_addys->next) {
-	    ia = (struct in_addr *) local_addys->data;
+	    struct in_addr *ia = (struct in_addr *) local_addys->data;
 
 	    if (ia == NULL)
 		    break;
@@ -347,7 +367,7 @@ gdm_auth_secure_display (GdmDisplay *d)
     fclose (af);
     if (af_gdm != NULL)
 	    fclose (af_gdm);
-    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+    ve_setenv ("XAUTHORITY", d->authfile, TRUE);
 
     if (GdmDebug)
 	    gdm_debug ("gdm_auth_secure_display: Setting up access for %s - %d entries", 
@@ -467,7 +487,7 @@ try_user_add_again:
 
 	locked = TRUE;
 
-	af = fopen (d->userauth, "a+");
+	af = gdm_safe_fopen_ap (d->userauth);
     }
 
     if (!af) {
@@ -594,11 +614,15 @@ gdm_auth_user_remove (GdmDisplay *d, uid_t user)
 	return;
     }
 
-    af = fopen (d->userauth, "a+");
+    af = gdm_safe_fopen_ap (d->userauth);
 
     if (!af) {
 	XauUnlockAuth (d->userauth);
 
+	gdm_error (_("%s: Cannot safely open %s"),
+		   "gdm_auth_user_remove",
+		   d->userauth);
+
 	g_free (d->userauth);
 	d->userauth = NULL;
 
