@@ -40,9 +40,9 @@
 
 /* External vars */
 extern gboolean GdmXdmcp;
-extern gint sessions;
+extern gint xdmcp_sessions;
 extern gint flexi_servers;
-extern gint pending;
+extern gint xdmcp_pending;
 extern GSList *displays;
 extern GdmConnection *fifoconn;
 extern GdmConnection *unixconn;
@@ -196,6 +196,8 @@ gdm_display_manage (GdmDisplay *d)
     d->slavepid = 0;
     gdm_sigchld_block_pop ();
 
+    d->managetime = time (NULL);
+
     /* Fork slave process */
     gdm_sigchld_block_push ();
     gdm_sigterm_block_push ();
@@ -370,11 +372,12 @@ gdm_display_dispose (GdmDisplay *d)
 	    flexi_servers --;
 
     if (d->type == TYPE_XDMCP) {
-	sessions--;
-	d->type = -1;
+	if (d->dispstat == XDMCP_MANAGED)
+		xdmcp_sessions--;
+	else if (d->dispstat == XDMCP_PENDING)
+		xdmcp_pending--;
 
-	if (d->dispstat == XDMCP_PENDING)
-		pending--;
+	d->type = -1;
     }
 
     if (d->name) {
@@ -389,6 +392,10 @@ gdm_display_dispose (GdmDisplay *d)
     g_free (d->hostname);
     d->hostname = NULL;
 
+    g_free (d->addrs);
+    d->addrs = NULL;
+    d->addr_count = 0;
+
     g_free (d->authfile);
     d->authfile = NULL;
 
