@@ -62,6 +62,7 @@
 				   restart display */
 #define DISPLAY_RUN_CHOOSER 30	/* Run chooser */
 #define DISPLAY_XFAILED 64	/* X failed */
+#define DISPLAY_GREETERFAILED 65 /* greeter failed (crashed) */
 #define DISPLAY_RESTARTGREETER 127 /* Restart greeter */
 #define DISPLAY_RESTARTGDM 128	/* Restart GDM */
 
@@ -129,7 +130,7 @@ enum {
  * to nothing */
 #define GDM_KEY_AUTOMATICLOGIN_ENABLE "daemon/AutomaticLoginEnable=true"
 #define GDM_KEY_AUTOMATICLOGIN "daemon/AutomaticLogin="
-#define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=true"
+#define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=false"
 #define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin"
 #define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_BINDIR "/gdmlogin"
 #define GDM_KEY_ADD_GTK_MODULES "daemon/AddGtkModules=false"
@@ -218,7 +219,7 @@ enum {
 #define GDM_KEY_ICONHEIGHT "gui/MaxIconHeight=128"
 
 #define GDM_KEY_BROWSER "greeter/Browser=false"
-#define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm"
+#define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm,rpm,nfsnobody,pcap"
 #define GDM_KEY_MINIMALUID "greeter/MinimalUID=100"
 #define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "nobody.png"
 #define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/faces/"
@@ -309,10 +310,17 @@ struct _GdmDisplay {
     gchar *name;
     gchar *hostname;
     struct in_addr addr;
+    struct in_addr *addrs; /* array of addresses */
+    int addr_count; /* number of addresses in array */
+    /* Note that the above may in fact be empty even though
+       addr is set, these are just extra addresses
+       (it could also contain addr for all we know) */
+
     guint8 dispstat;
     guint16 dispnum;
     guint8 servstat;
     time_t starttime;
+    time_t managetime;
     guint8 type;
     pid_t greetpid;
     pid_t servpid;
@@ -338,6 +346,8 @@ struct _GdmDisplay {
     time_t last_x_failed;
     int x_faileds;
 
+    gboolean try_different_greeter;
+
     gboolean disabled;
 
     gboolean logged_in; /* TRUE if someone is logged in */
