@@ -49,6 +49,7 @@ extern gchar *GdmPidFile;
 extern gboolean GdmDebug;
 extern GSList *displays;
 extern int gdm_xdmcpfd;
+extern gboolean GdmXdmcp;
 
 extern char **environ;
 
@@ -237,7 +238,7 @@ gdm_clearenv_no_lang (void)
 			envs = g_list_prepend (envs, g_strdup (env));
 	}
 
-	gnome_clearenv ();
+	ve_clearenv ();
 
 	for (li = envs; li != NULL; li = li->next) {
 		putenv (li->data);
@@ -262,7 +263,7 @@ gdm_clearenv (void)
 			envs = g_list_prepend (envs, g_strdup (env));
 	}
 
-	gnome_clearenv ();
+	ve_clearenv ();
 
 	for (li = envs; li != NULL; li = li->next) {
 		putenv (li->data);
@@ -294,7 +295,7 @@ gdm_restoreenv (void)
 {
 	GList *li;
 
-	gnome_clearenv ();
+	ve_clearenv ();
 
 	for (li = stored_env; li != NULL; li = li->next) {
 		putenv (g_strdup (li->data));
@@ -318,6 +319,7 @@ gdm_get_free_display (int start, uid_t server_uid)
 		GSList *li;
 		struct stat s;
 		char buf[256];
+		FILE *fp;
 
 		for (li = displays; li != NULL; li = li->next) {
 			GdmDisplay *dsp = li->data;
@@ -343,6 +345,32 @@ gdm_get_free_display (int start, uid_t server_uid)
 		}
 		close (sock);
 
+		/* if lock file exists and the process exists */
+		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
+		if (stat (buf, &s) == 0 &&
+		    ! S_ISREG (s.st_mode)) {
+			/* Eeeek! not a regular file?  Perhaps someone
+			   is trying to play tricks on us */
+			continue;
+		}
+		fp = fopen (buf, "r");
+		if (fp != NULL) {
+			char buf2[100];
+			if (fgets (buf2, sizeof (buf2), fp) != NULL) {
+				gulong pid;
+				if (sscanf (buf2, "%lu", &pid) == 1 &&
+				    kill (pid, 0) == 0) {
+					fclose (fp);
+					continue;
+				}
+
+			}
+			fclose (fp);
+
+			/* whack the file, it's a stale lock file */
+			unlink (buf);
+		}
+
 		/* if starting as root, we'll be able to overwrite any
 		 * stale sockets or lock files, but a user may not be
 		 * able to */
@@ -355,7 +383,7 @@ gdm_get_free_display (int start, uid_t server_uid)
 			}
 
 			g_snprintf (buf, sizeof (buf),
-				    "/tmp/.%d-lock", i);
+				    "/tmp/.X%d-lock", i);
 			if (stat (buf, &s) == 0 &&
 			    s.st_uid != server_uid) {
 				continue;
@@ -565,8 +593,8 @@ gdm_exec_wait (char * const *argv, gboolean no_display,
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
 		if (no_display) {
-			gnome_unsetenv ("DISPLAY");
-			gnome_unsetenv ("XAUTHORITY");
+			ve_unsetenv ("DISPLAY");
+			ve_unsetenv ("XAUTHORITY");
 		}
 		
 		execv (argv[0], argv);
@@ -672,14 +700,20 @@ pid_t
 gdm_fork_extra (void)
 {
 	pid_t pid;
-	gdm_sigchld_block_push ();
 
+	gdm_sigchld_block_push ();
 	gdm_sigterm_block_push ();
+
 	pid = extra_process = fork ();
 	if (pid < 0)
 		extra_process = 0;
-	gdm_sigterm_block_pop ();
+	else if (pid == 0)
+		/* unset signals here, and yet again
+		   later as the block_pop will whack
+		   our signal mask*/
+		gdm_unset_signals ();
 
+	gdm_sigterm_block_pop ();
 	gdm_sigchld_block_pop ();
 
 	if (pid == 0) {
@@ -691,6 +725,12 @@ gdm_fork_extra (void)
 		 * kill -(extra_process) to kill extra process and all it's
 		 * possible children */
 		setsid ();
+
+		/* Harmless in children, but in case we'd run
+		   extra processes from main daemon would fix
+		   problems ... */
+		if (GdmXdmcp)
+			gdm_xdmcp_close ();
 	}
 
 	return pid;
@@ -1103,7 +1143,6 @@ gdm_locale_to_utf8 (const char *text)
 {
 	GIConv cd;
 	char *out;
-	GError *error = NULL;
 
 	if (gdm_charset == NULL) {
 		return g_strdup (text);
@@ -1114,7 +1153,7 @@ gdm_locale_to_utf8 (const char *text)
 		return ascify (text);
 	}
 
-	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, &error);
+	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, NULL /* error */);
 	g_iconv_close (cd);
 	if (out == NULL) {
 		return ascify (text);
@@ -1137,13 +1176,236 @@ gdm_locale_from_utf8 (const char *text)
 		return ascify (text);
 	}
 
-	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, NULL);
+	out = g_convert_with_iconv (text, -1, cd, NULL, NULL, NULL /* error */);
 	g_iconv_close (cd);
 	if (out == NULL)
 		return ascify (text);
 	return out;
 }
 
+static GdmHostent *
+fillout_hostent (struct hostent *he_, struct in_addr *ia, const char *name)
+{
+	GdmHostent *he;
 
+	he = g_new0 (GdmHostent, 1);
+
+	he->addrs = NULL;
+	he->addr_count = 0;
+
+	/* Sometimes if we can't look things up, we could end
+	   up with a dot in the name field which would screw
+	   us up.  Weird but apparently possible */
+	if (he_ != NULL &&
+	    he_->h_name != NULL &&
+	    he_->h_name[0] != '\0' &&
+	    strcmp (he_->h_name, ".") != 0) {
+		he->hostname = g_strdup (he_->h_name);
+		he->not_found = FALSE;
+	} else {
+		he->not_found = TRUE;
+		if (name != NULL)
+			he->hostname = g_strdup (name);
+		else /* either ia or name is set */
+			he->hostname = g_strdup (inet_ntoa (*ia));
+	}
+
+	if (he_ != NULL && he_->h_addrtype == AF_INET) {
+		int i;
+		for (i = 0; ; i++) {
+			struct in_addr *ia_ = (struct in_addr *) (he_->h_addr_list[i]);
+			if (ia_ == NULL)
+				break;
+		}
+		he->addrs = g_new0 (struct in_addr, i);
+		he->addr_count = i;
+		for (i = 0; ; i++) {
+			struct in_addr *ia_ = (struct in_addr *) he_->h_addr_list[i];
+			if (ia_ == NULL)
+				break;
+			(he->addrs)[i] = *ia_;
+		}
+	}
+	return he;
+}
+
+GdmHostent *
+gdm_gethostbyname (const char *name)
+{
+	struct hostent *he_;
+
+	/* the cached address */
+	static GdmHostent *he = NULL;
+	static time_t last_time = 0;
+	static char *cached_hostname = NULL;
+
+	if (cached_hostname != NULL &&
+	    strcmp (cached_hostname, name) == 0) {
+		/* don't check more then every 5 seconds */
+		if (last_time + 5 > time (NULL))
+			return gdm_hostent_copy (he);
+	}
+
+	/* Find client hostname */
+	he_ = gethostbyname (name);
+	g_free (cached_hostname);
+	cached_hostname = g_strdup (name);
+
+	gdm_hostent_free (he);
+	he = fillout_hostent (he_, NULL, name);
+
+	last_time = time (NULL);
+	return gdm_hostent_copy (he);
+}
+
+GdmHostent *
+gdm_gethostbyaddr (struct in_addr *ia)
+{
+	struct hostent *he_;
+
+	/* the cached address */
+	static GdmHostent *he = NULL;
+	static time_t last_time = 0;
+	static struct in_addr cached_addr;
+
+	if (last_time != 0 &&
+	    memcmp (&cached_addr, ia, sizeof (struct in_addr)) == 0) {
+		/* don't check more then every 5 seconds */
+		if (last_time + 5 > time (NULL))
+			return gdm_hostent_copy (he);
+	}
+
+	/* Find client hostname */
+	he_ = gethostbyaddr ((gchar *) ia, sizeof (struct in_addr), AF_INET);
+	cached_addr = *ia;
+
+	gdm_hostent_free (he);
+	he = fillout_hostent (he_, ia, NULL);
+
+	last_time = time (NULL);
+	return gdm_hostent_copy (he);
+}
+
+GdmHostent *
+gdm_hostent_copy (GdmHostent *he)
+{
+	GdmHostent *cpy;
+
+	if (he == NULL)
+		return NULL;
+
+	cpy = g_new0 (GdmHostent, 1);
+	cpy->not_found = he->not_found;
+	cpy->hostname = g_strdup (he->hostname);
+	if (he->addr_count == 0) {
+		cpy->addr_count = 0;
+		cpy->addrs = NULL;
+	} else {
+		cpy->addr_count = he->addr_count;
+		cpy->addrs = g_new0 (struct in_addr, he->addr_count);
+		memcpy (cpy->addrs, he->addrs, sizeof (struct in_addr) * he->addr_count);
+	}
+	return cpy;
+}
+
+void
+gdm_hostent_free (GdmHostent *he)
+{
+	if (he == NULL)
+		return;
+	g_free (he->hostname);
+	he->hostname = NULL;
+
+	g_free (he->addrs);
+	he->addrs = NULL;
+	he->addr_count = 0;
+
+	g_free (he);
+}
+
+/* like fopen with "w" */
+FILE *
+gdm_safe_fopen_w (const char *file)
+{
+	int fd;
+	unlink (file);
+	fd = open (file, O_EXCL|O_CREAT|O_TRUNC|O_WRONLY, 0644);
+	if (fd < 0)
+		return NULL;
+	return fdopen (fd, "w");
+}
+
+/* like fopen with "a+" */
+FILE *
+gdm_safe_fopen_ap (const char *file)
+{
+	int fd;
+
+	if (access (file, F_OK) == 0) {
+#ifdef O_NOFOLLOW
+		fd = open (file, O_APPEND|O_RDWR|O_NOFOLLOW);
+#else
+		fd = open (file, O_APPEND|O_RDWR);
+#endif
+	} else {
+		/* doesn't exist, open with O_EXCL */
+		fd = open (file, O_EXCL|O_CREAT|O_RDWR, 0644);
+	}
+	if (fd < 0)
+		return NULL;
+	return fdopen (fd, "a+");
+}
+
+void
+gdm_reset_limits (void)
+{
+	struct rlimit unlim = { RLIM_INFINITY, RLIM_INFINITY };
+
+	/* Note: I don't really know which ones are really very standard
+	   and which ones are not, so I just test for them all one by one */
+
+#ifdef RLIMIT_CPU
+	setrlimit (RLIMIT_CPU, &unlim);
+#endif
+#ifdef RLIMIT_DATA
+	setrlimit (RLIMIT_DATA, &unlim);
+#endif
+#ifdef RLIMIT_FSIZE
+	setrlimit (RLIMIT_FSIZE, &unlim);
+#endif
+#ifdef RLIMIT_LOCKS
+	setrlimit (RLIMIT_LOCKS, &unlim);
+#endif
+#ifdef RLIMIT_MEMLOCK
+	setrlimit (RLIMIT_MEMLOCK, &unlim);
+#endif
+#ifdef RLIMIT_NOFILE
+	setrlimit (RLIMIT_NOFILE, &unlim);
+#endif
+#ifdef RLIMIT_OFILE
+	setrlimit (RLIMIT_OFILE, &unlim);
+#endif
+#ifdef RLIMIT_NPROC
+	setrlimit (RLIMIT_NPROC, &unlim);
+#endif
+#ifdef RLIMIT_RSS
+	setrlimit (RLIMIT_RSS, &unlim);
+#endif
+#ifdef RLIMIT_STACK
+	setrlimit (RLIMIT_STACK, &unlim);
+#endif
+#ifdef RLIMIT_CORE
+	setrlimit (RLIMIT_CORE, &unlim);
+#endif
+#ifdef RLIMIT_AS
+	setrlimit (RLIMIT_AS, &unlim);
+#endif
+#ifdef RLIMIT_VMEM
+	setrlimit (RLIMIT_VMEM, &unlim);
+#endif
+#ifdef RLIMIT_PTHREAD
+	setrlimit (RLIMIT_PTHREAD, &unlim);
+#endif
+}
 
 /* EOF */
