@@ -75,6 +75,34 @@ static GdmDisplay *d = NULL;
 static gboolean server_signal_notified = FALSE;
 static int server_signal_pipe[2];
 
+void
+gdm_server_whack_lockfile (GdmDisplay *disp)
+{
+	    char buf[256];
+
+	    /* X seems to be sometimes broken with its lock files and
+	       doesn't seem to remove them always, and if you manage
+	       to get into the weird state where the old pid now
+	       corresponds to some new pid, X will just die with
+	       a stupid error. */
+
+	    /* Yes there could be a race here if another X server starts
+	       at this exact instant.  Oh well such is life.  Very unlikely
+	       to happen though as we should really be the only ones
+	       trying to start X servers, and we aren't starting an
+	       X server on this display yet. */
+
+	    /* if lock file exists and it is our process, whack it! */
+	    g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", disp->dispnum);
+	    unlink (buf);
+
+	    /* whack the unix socket as well */
+	    g_snprintf (buf, sizeof (buf),
+			"/tmp/.X11-unix/X%d", disp->dispnum);
+	    unlink (buf);
+}
+
+
 /* Wipe cookie files */
 void
 gdm_server_wipe_cookies (GdmDisplay *disp)
@@ -168,7 +196,7 @@ gdm_server_reinit (GdmDisplay *disp)
 		XSetErrorHandler (ignore_xerror_handler);
 		XSetIOErrorHandler (exit_xioerror_handler);
 	       
-		gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+		ve_setenv ("XAUTHORITY", d->authfile, TRUE);
 		dsp = XOpenDisplay (d->name);
 
 		/* Now whack the server with a SIGHUP */
@@ -238,6 +266,8 @@ gdm_server_stop (GdmDisplay *disp)
 		    ve_waitpid_no_signal (servpid, 0, 0);
 	    gdm_sigchld_block_pop ();
 
+	    gdm_server_whack_lockfile (disp);
+
 	    gdm_debug ("gdm_server_stop: Server pid %d dead", (int)servpid);
     }
 
@@ -280,6 +310,7 @@ busy_ask_user (GdmDisplay *disp)
     }
 }
 
+/* Checks only output, no XFree86 v4 logfile */
 static gboolean
 display_xnest_no_connect (GdmDisplay *disp)
 {
@@ -323,9 +354,7 @@ display_busy (GdmDisplay *disp)
 		return FALSE;
 
 	while (fgets (buf, sizeof (buf), fp) != NULL) {
-		/* Note: this is probably XFree86 specific, and perhaps even
-		 * version 3 specific (I don't have xfree v4 to test this),
-		 * of course additions are welcome to make this more robust */
+		/* Note: this is probably XFree86 specific */
 		if (strstr (buf, "Server is already active for display")
 		    == buf) {
 			gdm_error (_("Display %s is busy. There is another "
@@ -340,12 +369,36 @@ display_busy (GdmDisplay *disp)
 	return FALSE;
 }
 
+/* if we find 'Log file: "foo"' switch fp to foo and
+   return TRUE */
+/* Note: assumes buf is of size 256 and is writable */ 
+static gboolean
+open_another_logfile (char buf[256], FILE **fp)
+{
+	if (strncmp (&buf[5], "Log file: \"", strlen ("Log file: \"")) == 0) {
+		FILE *ffp;
+		char *fname = &buf[5+strlen ("Log file: \"")];
+		char *p = strchr (fname, '"');
+		if (p == NULL)
+			return FALSE;
+		*p = '\0';
+		ffp = fopen (fname, "r");
+		if (ffp == NULL)
+			return FALSE;
+		fclose (*fp);
+		*fp = ffp;
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static int
 display_vt (GdmDisplay *disp)
 {
 	char *logname = g_strconcat (GdmLogDir, "/", d->name, ".log", NULL);
 	FILE *fp;
 	char buf[256];
+	gboolean switched = FALSE;
 
 	fp = fopen (logname, "r");
 	g_free (logname);
@@ -355,10 +408,19 @@ display_vt (GdmDisplay *disp)
 
 	while (fgets (buf, sizeof (buf), fp) != NULL) {
 		int vt;
-		/* Note: this is probably XFree86 specific, and perhaps even
-		 * version 3 specific (I don't have xfree v4 to test this),
-		 * of course additions are welcome to make this more robust */
-		if (sscanf (buf, "(using VT number %d)", &vt) == 1) {
+		char *p;
+
+		if ( ! switched &&
+		     /* this is XFree v4 specific */
+		    open_another_logfile (buf, &fp)) {
+			switched = TRUE;
+			continue;
+		} 
+		/* Note: this is probably XFree86 specific (works with
+		 * both v3 and v4 though */
+		p = strstr (buf, "using VT number ");
+		if (p != NULL &&
+		    sscanf (p, "using VT number %d", &vt) == 1) {
 			fclose (fp);
 			return vt;
 		}
@@ -452,7 +514,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     if ( ! gdm_auth_secure_display (d)) 
 	    return FALSE;
     gdm_slave_send_string (GDM_SOP_COOKIE, d->cookie);
-    gnome_setenv ("DISPLAY", d->name, TRUE);
+    ve_setenv ("DISPLAY", d->name, TRUE);
 
     if (pipe (server_signal_pipe) != 0) {
 	    gdm_error (_("%s: Error opening a pipe: %s"),
@@ -555,7 +617,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 		    /* In case we got a SIGCHLD */
 		    check_child_status ();
 
-		    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+		    ve_setenv ("XAUTHORITY", d->authfile, TRUE);
 		    for (i = 0;
 			 d->dsp == NULL &&
 			 d->servstat == SERVER_STARTED &&
@@ -570,7 +632,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			    /* In case we got a SIGCHLD */
 			    check_child_status ();
 		    }
-		    gnome_unsetenv ("XAUTHORITY");
+		    ve_unsetenv ("XAUTHORITY");
 		    if (d->dsp == NULL &&
 			/* Note: we could have still gotten a SIGCHLD */
 			d->servstat == SERVER_STARTED) {
@@ -661,6 +723,8 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 		kill (pid, SIGTERM) == 0)
 		    ve_waitpid_no_signal (pid, NULL, 0);
 	    gdm_sigchld_block_pop ();
+	    
+	    gdm_server_whack_lockfile (disp);
     }
 
     /* We will rebake cookies anyway, so wipe these */
@@ -928,6 +992,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
     gdm_sigterm_block_push ();
     pid = d->servpid = fork ();
+    if (pid == 0)
+	    gdm_unset_signals ();
     gdm_sigterm_block_pop ();
     gdm_sigchld_block_pop ();
     
@@ -1013,11 +1079,11 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
 	if (d->type == TYPE_FLEXI_XNEST) {
 		char *font_path = NULL;
-		gnome_setenv ("DISPLAY", d->xnest_disp, TRUE);
+		ve_setenv ("DISPLAY", d->xnest_disp, TRUE);
 		if (d->xnest_auth_file != NULL)
-			gnome_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
+			ve_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
 		else
-			gnome_unsetenv ("XAUTHORITY");
+			ve_unsetenv ("XAUTHORITY");
 
 		/* Add -fp with the current font path, but only if not
 		 * already among the arguments */
@@ -1057,11 +1123,11 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 		}
 		if (pwent->pw_dir != NULL &&
 		    g_file_test (pwent->pw_dir, G_FILE_TEST_EXISTS))
-			gnome_setenv ("HOME", pwent->pw_dir, TRUE);
+			ve_setenv ("HOME", pwent->pw_dir, TRUE);
 		else
-			gnome_setenv ("HOME", "/", TRUE); /* Hack */
-		gnome_setenv ("SHELL", pwent->pw_shell, TRUE);
-		gnome_unsetenv ("MAIL");
+			ve_setenv ("HOME", "/", TRUE); /* Hack */
+		ve_setenv ("SHELL", pwent->pw_shell, TRUE);
+		ve_unsetenv ("MAIL");
 
 		if (setgid (pwent->pw_gid) < 0)  {
 			gdm_error (_("%s: Couldn't set groupid to %d"), 
