@@ -137,7 +137,7 @@ check_update_error:
 		GtkWidget *setup_dialog = glade_helper_get
 			(xml, "setup_dialog", GTK_TYPE_WINDOW);
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (setup_dialog),
+			ve_hig_dialog_new (GTK_WINDOW (setup_dialog),
 						GTK_DIALOG_MODAL | 
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						GTK_MESSAGE_ERROR,
@@ -146,7 +146,8 @@ check_update_error:
 						  "trying to contact the "
 						  "login screens.  Not all "
 						  "updates may have taken "
-						  "effect."));
+						  "effect."),
+						/* avoid warning */ "%s", "");
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
@@ -863,10 +864,6 @@ setup_xdmcp_support (void)
 	gtk_widget_show (xdmcp_vbox);
 #endif /* HAVE_LIBXDMCP */
 
-	/* Why doesn't glade have this? */
-	gtk_label_set_use_markup (GTK_LABEL (GTK_BIN (xdmcp_toggle)->child),
-				  TRUE);
-
 	gtk_widget_set_sensitive (xdmcp_frame, 
 				  GTK_TOGGLE_BUTTON (xdmcp_toggle)->active);
 
@@ -1507,13 +1504,13 @@ install_ok (GtkWidget *button, gpointer data)
 	filename = g_strdup (gtk_file_selection_get_filename (fs));
 	if (filename == NULL) {
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (fs),
-						GTK_DIALOG_MODAL | 
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("No file selected"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			ve_hig_dialog_new (GTK_WINDOW (fs),
+					   GTK_DIALOG_MODAL | 
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   _("No file selected"),
+					   /* avoid warning */ "%s", "");
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		g_free (cwd);
@@ -1534,15 +1531,14 @@ install_ok (GtkWidget *button, gpointer data)
 
 	if (dir == NULL) {
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (fs),
-						GTK_DIALOG_MODAL | 
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("Not a theme archive\n"
-						  "Details: %s"),
-						error);
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			ve_hig_dialog_new (GTK_WINDOW (fs),
+					   GTK_DIALOG_MODAL | 
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   _("Not a theme archive"),
+					   _("Details: %s"),
+					   error);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		g_free (filename);
@@ -1553,19 +1549,38 @@ install_ok (GtkWidget *button, gpointer data)
 
 	if (dir_exists (theme_dir, dir)) {
 		char *fname = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+		char *s;
+		GtkWidget *button;
 		/* FIXME: if exists already perhaps we could also have an
 		 * option to change the dir name */
+		s = g_strdup_printf (_("Theme directory '%s' seems to be already "
+				       "installed, install again anyway?"),
+				     fname);
 		GtkWidget *dlg =
-			gtk_message_dialog_new
+			ve_hig_dialog_new
 			(GTK_WINDOW (fs),
 			 GTK_DIALOG_MODAL | 
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
 			 GTK_MESSAGE_QUESTION,
-			 GTK_BUTTONS_YES_NO,
-			 _("Theme directory '%s' seems to be already "
-			   "installed, install again anyway?"),
-			 fname);
+			 GTK_BUTTONS_NONE,
+			 s,
+			 /* avoid warning */ "%s", "");
 		g_free (fname);
+		g_free (s);
+
+		button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+		gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_NO);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+
+		button = gtk_button_new_from_stock ("_Install Anyway");
+		gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_YES);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+						 GTK_RESPONSE_YES);
+
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		if (gtk_dialog_run (GTK_DIALOG (dlg)) != GTK_RESPONSE_YES) {
 			gtk_widget_destroy (dlg);
@@ -1606,14 +1621,14 @@ install_ok (GtkWidget *button, gpointer data)
 
 	if ( ! success) {
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (fs),
-						GTK_DIALOG_MODAL | 
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("Some error occured when "
-						  "installing the theme"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			ve_hig_dialog_new (GTK_WINDOW (fs),
+					   GTK_DIALOG_MODAL | 
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   _("Some error occured when "
+					     "installing the theme"),
+					   /* avoid warning */ "%s", "");
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 	}
@@ -1685,6 +1700,7 @@ delete_theme (GtkWidget *button, gpointer data)
 	GtkTreeIter iter;
 	GValue value = {0, };
 	GtkWidget *dlg;
+	char *s;
 
 	setup_dialog = glade_helper_get (xml, "setup_dialog", GTK_TYPE_WINDOW);
 	theme_list = glade_helper_get (xml, "gg_theme_list",
@@ -1724,15 +1740,31 @@ delete_theme (GtkWidget *button, gpointer data)
 	dir = g_strdup (g_value_get_string (&value));
 	g_value_unset (&value);
 
-	dlg = gtk_message_dialog_new
+	s = g_strdup_printf (_("Do you really wish to remove theme '%s' from the system?"),
+			     name);
+	dlg = ve_hig_dialog_new
 		(GTK_WINDOW (setup_dialog),
 		 GTK_DIALOG_MODAL | 
 		 GTK_DIALOG_DESTROY_WITH_PARENT,
 		 GTK_MESSAGE_QUESTION,
-		 GTK_BUTTONS_YES_NO,
-		 _("Do you really wish to remove theme '%s' from the system?"),
-		 name);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+		 GTK_BUTTONS_NONE,
+		 s,
+		 /* avoid warning */ "%s", "");
+	g_free (s);
+
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_NO);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
+
+	button = gtk_button_new_from_stock ("_Remove Theme");
+	gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_YES);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
+	
+	gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+					 GTK_RESPONSE_YES);
+
 	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_YES) {
 		char *theme_dir = get_theme_dir ();
 		char *cwd = g_get_current_dir ();
@@ -1793,6 +1825,8 @@ setup_graphical_themes (void)
 
 	char *theme_dir = get_theme_dir ();
 
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (theme_list), TRUE);
+
 	selected_theme = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
 					       GDM_KEY_GRAPHICAL_THEME);
 
@@ -1899,6 +1933,7 @@ dialog_response (GtkWidget *dlg, int response, gpointer data)
 			return;
 		}
 	
+		/* HIG compliance? */
 		dlg = gtk_message_dialog_new
 			(GTK_WINDOW (setup_dialog),
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1947,6 +1982,24 @@ setup_gui (void)
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (dialog_response), NULL);
 
+	/* setup bold thingies */
+	glade_helper_tagify_label (xml, "greeter_cat_label", "b");
+	glade_helper_tagify_label (xml, "autologin_cat_label", "b");
+	glade_helper_tagify_label (xml, "timedlogin_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "sg_logo_cat_label", "b");
+	glade_helper_tagify_label (xml, "sg_background_cat_label", "b");
+	glade_helper_tagify_label (xml, "sg_misc_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "options_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "gg_preview_cat_label", "b");
+	glade_helper_tagify_label (xml, "gg_author_label", "b");
+	glade_helper_tagify_label (xml, "gg_desc_label", "b");
+	glade_helper_tagify_label (xml, "gg_copyright_label", "b");
+
+	glade_helper_tagify_label (xml, "enable_xdmcp", "b");
+
 	setup_xdmcp_support ();
 	setup_background_support ();
 	setup_greeter_backselect ();
@@ -1960,6 +2013,7 @@ setup_gui (void)
 	add_to_size_group (sg, "autologin_combo");
 	add_to_size_group (sg, "timedlogin_combo");
 	add_to_size_group (sg, "timedlogin_seconds");
+	g_object_unref (G_OBJECT (sg));
 
        	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	add_to_size_group (sg, "local_greeter_label");
@@ -1969,16 +2023,19 @@ setup_gui (void)
 	add_to_size_group (sg, "autologin_label");
 	add_to_size_group (sg, "timed_login_label");
 	add_to_size_group (sg, "timedlogin_seconds_label");
+	g_object_unref (G_OBJECT (sg));
 
        	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	add_to_size_group (sg, "greeter_table");
 	add_to_size_group (sg, "autologin_table");
 	add_to_size_group (sg, "timed_login_table");
+	g_object_unref (G_OBJECT (sg));
 
        	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 	add_to_size_group (sg, "sg_use_24_clock");
 	add_to_size_group (sg, "autologin");
 	add_to_size_group (sg, "timedlogin");
+	g_object_unref (G_OBJECT (sg));
 
 	setup_user_combo ("autologin_combo",
 			  GDM_KEY_AUTOMATICLOGIN);
@@ -2250,12 +2307,12 @@ main (int argc, char *argv[])
 	if ( ! DOING_GDM_DEVELOPMENT &&
 	     geteuid() != 0) {
 		GtkWidget *fatal_error = 
-			gtk_message_dialog_new (NULL /* parent */,
-						GTK_DIALOG_MODAL /* flags */,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("You must be the superuser (root) to configure GDM.\n"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (fatal_error), FALSE);
+			ve_hig_dialog_new (NULL /* parent */,
+					   GTK_DIALOG_MODAL /* flags */,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   _("You must be the superuser (root) to configure GDM."),
+					   /* avoid warning */ "%s", "");
 		if (RUNNING_UNDER_GDM)
 			setup_cursor (GDK_LEFT_PTR);
 		gtk_dialog_run (GTK_DIALOG (fatal_error));
