@@ -119,7 +119,7 @@ greeter_session_lookup (const char *saved_session)
 				   "future sessions?"),
 				 session_name (saved_session),
 				 session_name (default_session));	    
-	  save_session = greeter_query (msg);
+	  save_session = greeter_query (msg, _("Make _Default"), _("Just _Log In"));
 	  g_free (msg);
 	}
     }
@@ -150,7 +150,7 @@ greeter_session_lookup (const char *saved_session)
 				     session_name (session),
 				     session_name (saved_session),
 				     session_name (session));
-	      save_session = greeter_query (msg);
+	      save_session = greeter_query (msg, _("Make _Default"), _("Just For _This Session"));
 	    }
 	  else if (strcmp (session, "Xclients.desktop") != 0 &&
 		   strcmp (session, LAST_SESSION) != 0)
