@@ -52,7 +52,8 @@ working_command_exists (const char *commands)
 static void
 query_greeter_reboot_handler (void)
 {
-	if (greeter_query (_("Are you sure you want to reboot the machine?"))) {
+	if (greeter_query (_("Are you sure you want to reboot the machine?"),
+			   _("_Reboot"), GTK_STOCK_CANCEL)) {
 		closelog();
 		
 		_exit (DISPLAY_REBOOT);
@@ -63,7 +64,8 @@ query_greeter_reboot_handler (void)
 static void
 query_greeter_halt_handler (void)
 {
-	if (greeter_query (_("Are you sure you want to shut down the machine?"))) {
+	if (greeter_query (_("Are you sure you want to shut down the machine?"),
+			   _("Shut _Down"), GTK_STOCK_CANCEL)) {
 		closelog();
 
 		_exit (DISPLAY_HALT);
@@ -73,7 +75,8 @@ query_greeter_halt_handler (void)
 static void
 query_greeter_suspend_handler (void)
 {
-	if (greeter_query (_("Are you sure you want to suspend the machine?"))) {
+	if (greeter_query (_("Are you sure you want to suspend the machine?"),
+			   _("_Suspend"), GTK_STOCK_CANCEL)) {
 		/* suspend interruption */
 		printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_SUSPEND);
 		fflush (stdout);
@@ -125,6 +128,7 @@ greeter_system_append_system_menu (GtkWidget *menu)
 {
 	GtkWidget *w, *sep;
 	static GtkTooltips *tooltips = NULL;
+	gboolean add_reboot, add_halt, add_suspend;
 
 	/* should never be allowed by the UI */
 	if ( ! GdmSystemMenu ||
@@ -162,11 +166,17 @@ greeter_system_append_system_menu (GtkWidget *menu)
 				      NULL);
 	}
 
-	sep = gtk_separator_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), sep);
+	add_reboot = working_command_exists (GdmReboot);
+	add_halt = working_command_exists (GdmHalt);
+	add_suspend = working_command_exists (GdmSuspend);
 
+	if (add_reboot || add_halt || add_suspend) {
+		sep = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), sep);
+		gtk_widget_show (sep);
+	}
 
-	if (working_command_exists (GdmReboot)) {
+	if (add_reboot) {
 		w = gtk_menu_item_new_with_mnemonic (_("_Reboot the computer..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
@@ -176,11 +186,9 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
 				      _("Reboot your computer"),
 				      NULL);
-
-		gtk_widget_show (sep);
 	}
 
-	if (working_command_exists (GdmHalt)) {
+	if (add_halt) {
 		w = gtk_menu_item_new_with_mnemonic (_("Shut _down the computer..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
@@ -191,11 +199,9 @@ greeter_system_append_system_menu (GtkWidget *menu)
 				      _("Shut down your computer so that "
 					"you may turn it off."),
 				      NULL);
-
-		gtk_widget_show (sep);
 	}
 
-	if (working_command_exists (GdmSuspend)) {
+	if (add_suspend) {
 		w = gtk_menu_item_new_with_mnemonic (_("Sus_pend the computer..."));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 		gtk_widget_show (GTK_WIDGET (w));
@@ -205,8 +211,6 @@ greeter_system_append_system_menu (GtkWidget *menu)
 		gtk_tooltips_set_tip (tooltips, GTK_WIDGET (w),
 				      _("Suspend your computer"),
 				      NULL);
-
-		gtk_widget_show (sep);
 	}
 }
 
